#include <ncbi.h>
#include "blast.h"
#include "aabet.h"
#define WORDSIZE_MAX	8
/* ----------   build table of amino acid neighborhood words  -----------*/

static void	enumerate PROTO((int i, SCORE score));

static DFA	*dp;

static signed char
	wword[WORDSIZE_MAX];	/* candidate for critical word */

static SCORE
	best[WORDSIZE_MAX];	/* best[i] = smallest score of wword[0]..wword[i]
				   that can be extended to a critical word */
static char		*query;			/* pointer into query sequence */
static Uint4	nwords, nexact;
static int		alpha;
static int		W;
static SCORE	T;
static SCORE	(*matrx)[ALPHAVAL_MAX+1][ALPHAVAL_MAX+1];
static signed char	(*ord)[ALPHAVAL_MAX+1][ALPHASIZE_MAX];
static int	wm1;
static char	*funcname = "bld_table_aa";

DFA *
bld_table_aa(q_seq, wordsize, threshold, alphasize, matrix, maxcost, order,
numwords, numexact)
	char	*q_seq;
	int		wordsize;
	SCORE	threshold;
	int		alphasize;	/* size of the alphabet */
			/* max. cost of substituting for a character */
	SCORE	(*maxcost)[ALPHAVAL_MAX+1],
			/* matrix[i][j] == the cost of replacing i by j */
			(*matrix)[ALPHAVAL_MAX+1][ALPHAVAL_MAX+1];
			/* order[i][j] == the j-th best character to substitute for i */
	signed char	(*order)[ALPHAVAL_MAX+1][ALPHASIZE_MAX];
	Uint4	*numwords, *numexact;
{
	register int	i;
	register SCORE	score;

	if (wordsize > WORDSIZE_MAX)
		fatal("%s:  Word size is greater than WORDSIZE_MAX (%d)",
			funcname, WORDSIZE_MAX);
	W = wordsize;
	T = threshold;
	wm1 = W - 1;
	nwords = nexact = 0;
	alpha = alphasize;
	matrx = matrix;
	ord = order;

	dp = dfa_open(NULL, 0, (LETTER *)AAID_MIN, AAID_MAX-AAID_MIN+1, wordsize,
				ckalloc, 50*KBYTE, NULL, (DFA_HALTNONALPHAS|DFA_NOXLTAB));
	if (dp == NULL) {
		dfa_perror(dp, module);
		fatal(funcname);
	}

	for (score=0, i=0; i<wm1; ++i)
		score += (*matrx)[q_seq[i]][q_seq[i]];

	for (query = q_seq; query[wm1] != NULLB; query++) {
		best[wm1] = T;
		for (i = wm1; i > 0; --i)
			best[i-1] = best[i] - (*maxcost)[query[i]];
		enumerate(0, 0);

		/* Check the score of a perfect match */
		score += (*matrx)[query[wm1]][query[wm1]];
		if (score < T && score > 0) {
			/*
			Hit on a perfect match, even if its score is less than T.
			This was not done by the program when the JMB paper
			was written, but was a suggested enhancement which should
			yield better (lower) figures for the number of MSPs
			that are overlooked on average.
			*/
			++nexact;
			if (dfa_add(dp, (LETTER *)query, W, (mempint)(query+W)) == NULL) {
				dfa_perror(dp, module);
				fatal(funcname);
			}
		}
		score -= (*matrx)[query[0]][query[0]];
	}
	if (nwords + nexact == 0)
		fatal("%s: no satisfying words in query sequence to compare against database", funcname);
	if (dfa_close(dp) != 0) {
		dfa_perror(dp, module);
		fatal(funcname);
	}

	if (numwords != NULL)
		*numwords = nwords;
	if (numexact != NULL)
		*numexact = nexact;

	return dp;
}

/* enumerate - try all letters in position i of candidate for a critical word;
	score is for wword[0]..wword[i-1] */
static void
enumerate(i, score)
	register int		i;
	register SCORE	score;
{
	/* possible letters by decreasing weight */
	register signed char	*o = (*ord)[query[i]];
	register int	c;
	register SCORE	new_score;

	for (c = 0; c < alpha; ++c) {
		new_score = score + (*matrx)[query[i]][wword[i] = o[c]];
		if (new_score < best[i])
			break;
		if (i < wm1)
			enumerate(i+1, new_score);
		else {
			++nwords;
			if (dfa_add(dp, (LETTER *)wword, W, (mempint)(query + W)) == NULL) {
				dfa_perror(dp, module);
				fatal(funcname);
			}
		}
	}
}
