#include <ncbi.h>
#include "blast.h"

/*
	cutoffs.c
	
	Calculate the cutoff score, S, and the highest expected score.

	WRG
*/
void
cutoffs(S, E, highs, Lambda, K, H, dblen, qlen)
	SCORE	*S;		/* cutoff score */
	double	*E;		/* Expected number of HSPs scoring above S */
	SCORE	*highs;	/* Expected high score */
	double	Lambda, K, H; /* parameters for Karlin statistics */
	unsigned long	dblen;	/* length of database */
	unsigned long	qlen;	/* length of query sequence */
{
	SCORE	s = *S, es;
	double	e = *E;

	/*
	Calculate a cutoff score, S, from the Expected
	(or desired) number of reported HSPs, E.
	*/
	if (e > 0.) {
		es = etos(e, Lambda, K, H, dblen, qlen);
		/*
		Pick the larger cutoff of the user's choice
		vs. that calculated from the value of E.
		*/
		if (es >= s)
			s = es;
	}

	/*
		Re-calculate E from the cutoff score, thus obtaining
		the effective value of E given S.
	*/
	*S = s;
	*E = stoe(s, Lambda, K, H, dblen, qlen);

	/* Calculate the expected high score */
	*highs = ehighs(Lambda, K, H, dblen, qlen);

	return;
}
