/*
	gb2fasta.c

	Convert a list of GenBank-format sequence files into FASTA format
	(for subsequent processing by PRESSDB and use by BLASTN).

	Usage:  gb2fasta gbpri.seq gbrod.seq gbmam.seq ... > genbank.total

	Suggested alternative usage:

	gb2fasta -l GenPept genpept.seq > genpept

	Tested successfully on GenBank Release 64.0

	7/22/90 WRG
*/
#include <ncbi.h>
#include <ncbilib.h>

FILE	*fp;
char	buf[256];

char	*label = "GenBank";

main(ac, av)
	int		ac;
	char	**av;
{
	int		i;
	register char	*cp, ch;

	i = 1;

	if (strcmp(av[1], "-l") == 0) {
		/* extract a label to apply to each sequence name */
		label = str_dup(av[2]);
		i = 3;
	}

	for (; i<ac; ++i) {
		if (strcmp(av[i], "-") == 0) /* Read from stdin? */
			fp = fdopen(0, "rb");
		else
			fp = fopen(av[i], "rb");
		if (fp == NULL)
			goto NextFile;


NextSeq:
		do {
			if (fgets(buf, sizeof(buf)-1, fp) == NULL)
				goto NextFile;
		} while (buf[0] != 'L' || strncmp(buf, "LOCUS", 5));
		cp = buf+5;
		while (isspace(*cp))
			++cp;
		putchar('>');
		while (!isspace(ch = *cp)) {
			putchar(ch);
			++cp;
		}
		printf(" (%s) ", label);

		do {
			if (fgets(buf, sizeof(buf)-1, fp) == NULL)
				goto NextFile;
		} while (buf[0] != 'D' || strncmp(buf, "DEFINITION", 10));

		cp = buf+10;
		while (isspace(*cp))
			++cp;
		cp[strlen(cp)-1] = NULLB;
		printf("%s", cp);
		while (fgets(buf, sizeof(buf)-1, fp) != NULL &&
					buf[0] == ' ') {
			cp = buf+1;
			while (isspace(*cp))
				++cp;
			cp[strlen(cp)-1] = NULLB;
			putchar(' ');
			printf("%s", cp);
		}
		putchar('\n');

		while (fgets(buf, sizeof(buf)-1, fp) != NULL &&
				(buf[0] != 'O' || strncmp(buf, "ORIGIN", 6)) );

		while (fgets(buf, sizeof(buf)-1, fp) != NULL && buf[0] != '/') {
			cp = buf+10;
			while ((ch = *cp++) != NULLB)
				if (isalpha(ch))
					putchar(TOUPPER(ch));
			putchar('\n');
		}

		goto NextSeq;
		

NextFile:
		if (fp != NULL)
			fclose(fp);
		continue;
	}
}
