#include <ncbi.h>
#include "blast.h"

#define PAGE_WIDTH	80

#define MULTI_CHR	'='
#define SINGLE_CHR	':'
#define VAXIS_CHR	'|'

void
print_hist(score_hist_dim, score_hist, S, T, lambda, K, H, qlen, dblen, nbr_seqs)
	SCORE	score_hist_dim;
	long	*score_hist;
	SCORE	S, T/* call with T==0 if there is no neighborhood to speak of */;
	double	lambda, K, H;
	long	qlen, dblen, nbr_seqs;
{
	register long	i;
	long	score_hist_max, score_hist_stop;
	unsigned long	observed;
	int		j, ncols, plotcols, npercol;
	Boolean		hadnonzero = FALSE;
	double	C, xqlen, expect, old_expect, cmpexpect;

	for (i=score_hist_max=observed=0; i<=score_hist_dim; ++i) {
		observed += score_hist[i];
		if (score_hist[i] > score_hist_max)
			score_hist_max = score_hist[i];
	}

	score_hist_stop = score_hist_dim;
	for (i=score_hist_dim-1; i>=0; --i) {
		if (score_hist[i] != 0) {
			score_hist[score_hist_stop = i+1] = score_hist[score_hist_dim];
			break;
		}
	}

	plotcols = PAGE_WIDTH - (12+1); /* One extra in case of auto-linefeed */
	npercol = ceil((double)score_hist_max/(double)plotcols);
	npercol = MAX(npercol, 1); /* at least one sequence per mark */

	xqlen = qlen;
	C = K * xqlen * dblen;
	old_expect = HUGE_VAL;
	expect = C * exp(-lambda * T);
#ifdef REFINED_STATS
	expect *= MAX(xqlen - lambda*T/H, 1.) / xqlen;
#endif
	expect = MIN(1000., expect);
	expect = MIN(expect, 0.01 * nbr_seqs); /* predict extreme values only */
	cmpexpect = exp(log(10.) * floor(log10(expect)));


	printf("\n\n    Highest-scoring Hit Extension vs. Count of Database Sequences\n\n");
	printf("        Histogram units:    %c %d Sequence%s",
				MULTI_CHR, npercol, (npercol == 1 ? "" : "s"));
	if (npercol > 1)
		printf("     %c less than %d sequences\n", SINGLE_CHR, npercol);
	else
		putchar('\n');
	putchar('\n');

	printf("Highest\n");
	printf("Score\n");
	printf("   |  Count-->\n");
	printf("   V\n");

	for (i=0; i<=score_hist_stop; observed -= score_hist[i++]) {
		if (i < T && i > 0 && !hadnonzero) {
			if (score_hist[i] == 0)
				continue;
			printf(" Low-scoring perfect match word hits\n");
			hadnonzero = TRUE;
		}
		if (i == T)
			printf(" Neighborhood word score threshold, T = %d\n", T);
		expect = C * exp(-lambda * i);
#ifdef REFINED_STATS
		expect *= MAX(xqlen - lambda*i/H, 1.) / xqlen;
#endif
		if (i == S) {
			printf(
			">>>>>>>>>>>>>>  Expect = %#0.2lg, Observed = %s  <<<<<<<<<  Cutoff Score %s\n",
			expect, lu2s(observed), lu2s(S));
			if (expect < cmpexpect && old_expect > cmpexpect)
				cmpexpect /= 10.;
		}
		else
			if (expect < cmpexpect && old_expect > cmpexpect) {
				cmpexpect /= 10.;
				printf("--------------  Expect = %#0.2lg, Observed = %s  -----------\n",
					expect, lu2s(observed));
			}

		if (i == score_hist_stop)
			putchar('>');
		else
			putchar(' ');
		printf("%3d%6d %c", i, score_hist[i], VAXIS_CHR);
		ncols = score_hist[i]/npercol;
		for (j=0; j<ncols; ++j)
			putchar(MULTI_CHR);
		if (j == 0 && score_hist[i] > 0)
			putchar(SINGLE_CHR);
		putchar('\n');
		old_expect = expect;
	}
}
