/* -----------------------   read the compressed database  -------------------*/

initdb(dbname)
	char	*dbname;
{
	int		many;
	char 	name[FILENAME_MAX];
	long	cleaned;
	off_t	dbsize;
	int		once = 0;
	char	*dirname = CWDSTR;
#ifdef USE_SHM
	int		i;
	int		dbexists;
#endif

RETRY:
	if (*dirname != NULLB)
		(void) sprintf(dbfname, "%s%c%s", dirname, DIRDELIMCHR, dbname);
	else
		STRNCPY(dbfname, dbname, sizeof(dbfname));
	sfile = fopen(dbfname, "r");
	if (sfile == NULL) {
		if ((dirname = getenv("BLASTN")) == NULL)
			dirname = BLASTN_DIR;
		if (++once == 1)
			goto RETRY;
		fatal("database not found");
	}
	dbsize = sys_filesize(name);
    if (dbsize == 0)
        fatal("compressed database file is empty");
	(void) sprintf(name, "%s.tab", dbfname);
	tfile = ckopen(name, "r");
	(void) fread((char *)&dbformat, sizeof(dbformat), 1, tfile);
	if (dbformat != NTFORMAT)
		fatal("Incorrect database format");
	if (fread((char *)&line_len, sizeof(line_len), 1, tfile) != 1 ||
	   fread((char *)&nbr_seqs, sizeof(nbr_seqs), 1, tfile) != 1 ||
	   fread((char *)&maxlen, sizeof(maxlen), 1, tfile) != 1 ||
	   fread((char *)&totdblen, sizeof(totdblen), 1, tfile) != 1 ||
	   fread((char *)&c_len, sizeof(c_len), 1, tfile) != 1 ||
	   fread((char *)&clean_count, sizeof(clean_count), 1, tfile) != 1 ||
		c_len != dbsize )
		fatal("corrupted database file");
	if (c_len > CDBLEN_MAX)
		fatal("database is too large; raise CDBLEN_MAX and recompile");

	(void) sprintf(name, "%s.cseq", dbfname);
#ifdef USE_SHM
	/* Load the database into shared memory, but only if dbname is "genbank" */
	if (str_eq(module, "blastn") && str_casecmp(dbname, "genbank") == 0) {
		dbexists = shmexists(BLASTNKEY, c_len+2);
		_c_seq = (PACK_TYPE *)shmalloc(BLASTNKEY, 0666, c_len+2);
		if (_c_seq == NULL)
			fatal("unable to get shared memory segment for database");
    	c_seq = _c_seq + 1;
		if (!dbexists) {
			(void) printf("Reading database into shared memory...\n");
			(void) fflush(stdout);
			if (fread((char *)c_seq, c_len, 1, cfile) != 1)
				fatal("Cannot read compressed database sequence file.");
		}
	}
	else
#endif
	{
		_c_seq = (PACK_TYPE *)ckalloc(c_len+2);
		if (_c_seq == NULL)
			fatal("unable to allocate %s bytes of memory for the database",
				lu2s(c_len + 2));
    	c_seq = _c_seq + 1;
		cfile = ckopen(name, "r");
		(void) printf("Reading database into memory...\n");
		(void) fflush(stdout);
		if (fread((char *)c_seq, c_len, 1, cfile) != 1)
			fatal("Cannot read compressed database sequence file.");
	}

#ifdef CLEAN8MERS
	if (do_clean)
		for (i = 0; i < clean_count; ++i) {
			if (fread((char *)&cleaned, sizeof(cleaned), 1, tfile) != 1)
				fatal("List of database 8-mers to be cleaned is corrupted");
			bucket[cleaned] = 1;
		}
	else
#else
	/* skip over the list of cleaned 8-mers */
	if (clean_count > 0)
		(void) fseek(tfile, clean_count * sizeof(cleaned), 1);
#endif

	many = nbr_seqs + 1;
	cseq_beg = (long *)ckalloc(sizeof(*cseq_beg)*many);
	seq_beg = (long *)ckalloc(sizeof(*seq_beg)*many);
	header_beg = (long *)ckalloc(sizeof(*header_beg)*many);
	if (fread((char *)cseq_beg, sizeof(*cseq_beg), many, tfile) != many ||
	   fread((char *)seq_beg, sizeof(*seq_beg), many, tfile) != many ||
	   fread((char *)header_beg, sizeof(*header_beg), many, tfile) != many)
			fatal("improper compressed database file");

	/* append artificial trailer (sentinel bytes) to halt DFA search */
	c_seq[c_len] = c_seq[c_len+1] = NT_MAGIC_BYTE;
	if (Neff == 0)
		Neff = totdblen;
}
