#include <ncbi.h>
#include <ncbilib.h>
#include <sys/lock.h>
#include <sys/file.h>
#include <sys/termio.h>
#include <signal.h>

#include "shmutil.h"

char	*module;

Boolean	verbose;	/* be verbose about the files being read into memory */
Boolean	lockfile;	/* lock data pages in real memory */
Boolean	daemonize;	/* fork and detach from controlling tty */
int		nfiles;		/* no. of files loaded into memory */

Signal_t	sighup(), sigterm();

int		argc, argbeg;
char	**argv;
int		mode = 0666;	/* mode for creating shared memory segments */
Boolean	bg;		/* TRUE ==> running in background */

main(ac, av)
	int		ac;
	char	**av;
{
	char	*memfile;
	FILE	*fp;
	struct stat	sbuf;
	int		i, c, childpid, pid;
	Boolean	errflg = FALSE;

	opterr = 0; /* disable getopt()'s error messages */

	module = basename(av[0]);

	argc = ac;
	argv = av;

	while ((c = getopt(ac, av, "vldm:C:")) != -1)
		switch (c) {
			case 'v':
				verbose = TRUE;
				break;
			case 'l':
				lockfile = TRUE;
				break;
			case 'd':
				daemonize = TRUE;
				break;
			case 'm':
				if (*optarg == '0')
					if (sscanf(optarg, "%o", &mode) != 1)
						usage();
					else;
				else
					if (sscanf(optarg, "%d", &mode) != 1)
						usage();
				break;
			case 'C':
				if (chdir(optarg) == -1) {
					perror(optarg);
					exit(errno);
				}
				break;
			case '?':
				errflg=TRUE;
		}

	if (errflg)
		usage();

	for (argbeg = optind; optind < ac; ++optind, ++nfiles) {

		if (verbose)
			(void) printf("%s: loading file \"%s\"\n", av[optind]);
		if (shm_loadfile(av[optind], mode, lockfile) == NULL) {
			--nfiles;
			*av[optind] = NULLB;
		}
	}

	if (nfiles == 0)
		usage();

	bg = (signal(SIGINT, SIG_IGN) == SIG_IGN);
	if (daemonize) {
		childpid = fork();
		if (childpid < 0) {
			perror(module);
			exit(1);
		}
		if (childpid > 0)
			exit(0);

		pid = getpid();

		/* Dissociate from controlling tty */
		if (setpgrp(0, pid) == -1)
			exit(errno);

		if ((i = open("/dev/tty", O_RDWR)) >= 0) {
			ioctl(i, TIOCNOTTY, (char *)NULL);
			close(i);
		}
	}

	if (daemonize || !bg)
		signal(SIGINT, sigterm);
	signal(SIGHUP, sighup);
	signal(SIGQUIT, sigterm);
	signal(SIGTERM, sigterm);

	if (lockfile && geteuid() != 0)
		(void) fprintf(stderr,
				"%s:  could not lock data in memory\n", module);

	/* wait forever (or until killed) */
	for (;;)
		pause();

	/*NOTREACHED*/
}


usage()
{
	(void) fprintf(stderr,
		"Usage:  %s [-v] [-l] [-d] [-m mode] [-C dirname] filename\n", module);
	exit(1);
}

Signal_t
sighup()
{
	register int	i;

	for (i = argbeg; i < argc; ++i)
		shm_dropfile(argv[i]);

	if (!bg)
		exit(0);

	for (i = argbeg; i < argc; ++i)
		shm_loadfile(argv[i], mode, lockfile);
}

Signal_t
sigterm()
{
	register int	i;

	for (i = argbeg; i < argc; ++i)
		shm_dropfile(argv[i]);
	exit(0);
}
