#include <ncbi.h>
#include <ncbilib.h>
#include "blast.h"


MFILE far *
mfil_open(name, mode)
	char far	*name, *mode;
{
	MFILE far	*mfp;
	char	*shm_attfile();

	mfp = (MFILE far *)malloc(sizeof(MFILE));
	if (mfp == NULL)
		return NULL;
	
	mfp->fp = NULL;

#ifdef USE_SHM
	mfp->offset = 0;
	mfp->sp = shm_attfile(name, mode);
	if (mfp->sp == NULL) {
#endif
		mfp->fp = fopen(name, "r");
		if (mfp->fp == NULL) {
			(void) free((char far *)mfp);
			return NULL;
		}
#ifdef USE_SHM
	}
#endif

	mfp->filesize = sys_filesize(name);

	return mfp;
}


int
mfil_close(mfp)
	MFILE far	*mfp;
{
	if (mfp->sp != NULL)
		(void) shm_detach(mfp->sp);
	else
		if (mfp->fp != NULL)
			(void) fclose(mfp->fp);
	mfp->sp = NULL;
	mfp->fp = NULL;
	(void) free((char far *)mfp);
	return 0;
}


/*
	mfil_get() -- return a pointer to an area of storage of length 'len'
	that is populated with data from the current MFILE pointer for length
	'len'.  The current MFILE pointer is then advanced by 'len'.

	If MFILE is not in shared memory, then the memory area is allocated
	with malloc() and the data is read into that memory from disk.

	WARNING:  data alignment problems (bus errors) may result from improper
	use of this function!
*/
char far *
mfil_get(mfp, len)
	MFILE far	*mfp;
	size_t	len;
{
	register size_t	off;
	register char far	*cp;

#ifdef USE_SHM
	if (mfp->sp != NULL) {
		off = mfp->offset;
		if (off + len > mfp->filesize)
			len = mfp->filesize - off;
		mfp->offset = off + len;
		return mfp->sp + off;
	}
#endif

	cp = ckalloc(len);
	if (fread(cp, len, 1, mfp->fp) != 1) {
		(void) free(cp);
		return NULL;
	}
	return cp;
}

/*
	mfil_dup() -- copy data from the memory file into the provided buffer
	for length 'len'.
*/
size_t
mfil_dup(mfp, buf, len)
	MFILE far	*mfp;
	char far	*buf;
	register size_t	len;
{
#ifdef USE_SHM
	register size_t	tmp;

	if (mfp->sp != NULL) {
		if ((tmp = mfp->offset + len) > mfp->filesize) {
			len = mfp->filesize - mfp->offset;
			tmp = mfp->filesize;
		}
		mem_cpy(buf, mfp->sp + mfp->offset, len);
		mfp->offset = tmp;
		return len;
	}
#endif

	return fread(buf, 1, len, mfp->fp);
}


/*
	mfil_seek() -- reposition the memory file pointer
*/
int
mfil_seek(mfp, offset, ptrname)
	MFILE far	*mfp;
	long	offset;
	int		ptrname;
{
	register long	i;

#ifdef USE_SHM
	if (mfp->sp != NULL) {
		switch (ptrname) {
			case SEEK_SET: /* relative to beginning */
				if (offset < 0 || offset > mfp->filesize)
					return -1;
				mfp->offset = offset;
				break;
			case SEEK_CUR: /* relative to current position */
				i = mfp->offset + offset;
				if (i > mfp->filesize || i < 0)
					return -1;
				mfp->offset = i;
				break;
			case SEEK_END: /* relative to end of file */
				if (mfp->filesize < offset || offset < 0)
					return -1;
				mfp->offset = mfp->filesize - offset;
				break;
			default:
				return -1;
		}
		return 0;
	}
#endif

	return fseek(mfp->fp, offset, ptrname);
}
