
static void	extend_aa PROTO((MPPROTOC char far *s, char far *q));

/* -----------------------   search a database sequence   --------------------*/
static void
search_aa(MP)
	MPDEF
{
	register STATE far	*state;
	register dfa_accept far	*ap;
	register unsigned char far	*s;

	MPPTR cutoff = S;

#ifndef BLAST3
Repeat:
#endif
	state = dfa_state0(dp);
	s = (unsigned char *)MPPTR s_seq;

	for (;;) {
		/*
			The machine "accepts" when a word hit is encountered,
			and "halts" on the null terminator or any other non-alphabetic
			character encountered
		*/
		while (!IsAccepting(state = state->next[*s++]))
			;

		/* Has finite-state machine halted (e.g., end of sequence)? */
		if ((ap = (dfa_accept *)Unmunged(state)) == NULL)
			break; /* yes */

		/* Found at least one hit... */

		/* Does list of hit positions contain more than one item? */
		if (!IsMunged(state = ap->ptr.retState))
			ap = ap->val.next; /* yes, ap->val.next points to head of list */

		/* Always Unmunge state, to avoid a costly jump if it isn't */
		state = (STATE *)Unmunged(state);

		/* Extend each hit in the linked list */
		do
			extend_aa(MPC (char far *)s, (char far *)ap->val.patID);
		while (!IsMunged(ap = ap->ptr.chain));
	}

	/* Clear record of diagonal hit positions */
	while (MPPTR diag_top > MPPTR diag_stack)
		MPPTR diag_level[*--(MPPTR diag_top)] = 0;

#ifndef BLAST3
	/*
		If at least one HSP was found, search the current
		database sequence again using a lower cutoff score.
		(Caution:  a low threshold, T, should be in effect
		for there to be a reasonable chance of seeing many
		low-scoring HSPs; for instance, W=4 T=13 is recommended
		for use with the PAM120 substitution matrix provided).
	*/
	if (MPPTR curHSP > 0 && MPPTR cutoff != S2) {
		MPPTR cutoff = S2;
		goto Repeat;
	}
#endif
}

/* extend_aa - extend a word-sized hit to a longer match */
static void
extend_aa(MPC s_pos, q_pos)
	MPDEF
	register char far	*s_pos;
	register char far	*q_pos;
{
	register char far	*q, *s;
	register long	sum;
	register int	x;
	register long	score;
	register char far	*q_beg, *q_end;
	register int	diag;

	MPPTR numhits++;
	x = q_pos - q_seq;
	if (MPPTR diag_level[diag = (s_pos - MPPTR s_seq) - x] > x)
		return;
	if (MPPTR diag_level[diag] == 0)
		*MPPTR diag_top++ = diag;

	/* Find where positive scoring starts & ends within the word hit */
	q_beg = q = q_pos - W;
	s = s_pos - W;
	score = sum = 0;
	do
		if ((sum += matrx[*q++][*s++]) > score) {
			score = sum;
			q_end = q;
		}
		else
			if (sum <= 0) {
				sum = 0;
				q_beg = q;
			}
	while (q < q_pos);

	if ((x = -score) < minusX)
		x = minusX;

	/* extend to the left */
	s = s_pos - (q_pos - (q = q_beg));
	sum = 0;
	do
		if ((sum += matrx[*--q][*--s]) > 0) {
			q_beg = q;
			score += sum;
			/* unroll one iteration of the do-while loop */
			if ((sum = matrx[*--q][*--s]) > 0) {
				q_beg = q;
				score += sum;
				sum = 0;
			}
			if ((x = -score) < minusX)
				x = minusX;
		}
	while (sum >= x); /* fails on null prefix of either sequence */

	/* extend to the right */
	s = s_pos + ((q = q_end) - q_pos);
	sum = 0;
	do
		if ((sum += matrx[*q++][*s++]) > 0) {
			q_end = q;
			score += sum;
			/* unroll one iteration of the do-while loop */
			if ((sum = matrx[*q++][*s++]) > 0) {
				q_end = q;
				score += sum;
				sum = 0;
			}
			if ((x = -score) < minusX)
				x = minusX;
		}
	while (sum >= x); /* fails on null terminator of either sequence */

	if (score < MPPTR cutoff) {	/* Score is not reportable */
		/* Record how far this diagonal has been traversed */
		MPPTR diag_level[diag] = (q - q_seq) + W;
		++MPPTR numfailed;
		if (MPPTR high_score < score)
			MPPTR high_score = score;
		return;
	}

	/* Score is reportable.  Now examine the diagonal rigorously... */
	/* (This was not done by the program described in the JMB paper) */

	/* Mark this diagonal (in advance) as having been completely examined */
	MPPTR diag_level[diag] = INT_MAX;

	/* Start at the very beginning of the diagonal */
	s = MPPTR s_seq;
	q = q_seq;
	if (diag > 0)
		s += diag;
	else
		q -= diag;

	q_beg = q;
	score = sum = 0;
	for (;;) {
		if ((sum += matrx[*q++][*s++]) <= 0) {
			if (score > 0) {
				q = q_end;
				s = MPPTR s_seq + (q_end - q_seq) + diag;
				if (score >= MPPTR cutoff) {
					struct hsp	h;

					if (MPPTR high_score < score)
						MPPTR high_score = score;

					/* Record the HSP */
					h.score = score;
					h.seq_nbr = MPPTR current;
					h.len = q_end - q_beg;
					h.s_pos = (h.q_pos = q_beg - q_seq) + diag;
#ifdef TBLASTN
					h.frame = MPPTR frame;
#endif
					ckoverlap(MPC &h);
				}
			}
			else
				if (sum < SCORE_MIN/4)
					break; /* Hit a sentinel byte */
			q_beg = q_end = q;
			score = sum = 0;
		}
		else
			if (sum > score) {
				score = sum;
				q_end = q;
			}
	}
}
