static void	extend_nt PROTO((MPPROTOC PACK_TYPE *c, long q_pos, unsigned int left, unsigned int right, char *qq_seq));

/* ----------------------- scan a nucleotide database --------------------*/
static void
search_nt(MP)
	MPDEF
{
	register STATE	*state;
	register PACK_TYPE	*c = MPPTR c_seq - 1;
	register struct dfa_accept	*ap;
	register long	q_pos;
	register char	*q;
	register PACK_TYPE	p;
	register unsigned int	left, right;
	PACK_TYPE	*end = MPPTR c_seq + MPPTR c_len;
	char	*qq_seq;

	state = dfa_state0(dp);

	for (;;) {
		while (!IsAccepting(state = state->next[*++c]))
			;

		ap = (dfa_accept *)Unmunged(state);
		if (!IsMunged(state = ap->ptr.retState))
			ap = ap->val.next; /* More than one item in the list */

		/* Always Unmunge state, simply to avoid a costly jump if it isn't */
		state = (STATE *)Unmunged(state);
	
		do {
			if ((q_pos = ap->val.patID) > 0) {
				q = qq_seq = qq_for;
				/* the location of this test is the reason MAGIC_NUMBER must
				be a positive integer */
				if (--q_pos == (MAGIC_NUMBER-1)) {
					if (c >= end) /* end of database? */
						return;
					continue;
				}
			}
			else {
				q = qq_seq = qq_rev;
				q_pos = -q_pos - 1;
			}

			++MPPTR numhits8;

			/* how many positions left does the match extend? */
			q += q_pos - 1;
			p = c[-WORDSIZE_MIN/(CHAR_BIT/NBPN)];
			if (BASE3(p) != *q)
				left = 0;
			else
				if (BASE2(p) != *--q)
					left = 1;
				else
					if (BASE1(p) != *--q)
						left = 2;
					else
						if (BASE0(p) != *--q)
							left = 3;
						else
							left = 4;
		
			/* how many positions right does the match extend? */
			q = qq_seq + q_pos + WORDSIZE_MIN;
			p = c[1];
			if (BASE0(p) != *q)
				right = 0;
			else
				if (BASE1(p) != *++q)
					right = 1;
				else
					if (BASE2(p) != *++q)
						right = 2;
					else
						if (BASE3(p) != *++q)
							right = 3;
						else
							right = 4;

			if (left + right >= WmWM)
				extend_nt(MPC c, q_pos, left, right, qq_seq);

		} while (!IsMunged(ap = ap->ptr.chain));
	} /* for (;;) */
}

/* extend_nt - extend word-sized hit to longer match */
static void
extend_nt(MPC c, q_pos, left, right, qq_seq)
	MPDEF
	PACK_TYPE	*c;
	long	q_pos;
	register unsigned int	left, right;
	char	*qq_seq;
{
	register char	*q, *d, *dlim;
	register int	sum, x, n = N, m = M;
	int		diagentry;
	int		c_pad, score;
	long	diag;
	PACK_TYPE	*cbp, *cend;
	char	*q_beg, *q_end;
	long	cbyte;
	long	c_end = MPPTR c_len-1; /* offset of last byte in compressed seq */
	int		limit;
	char	db_piece[CHAR_BIT/NBPN];

	/* check whether ends of query sequence were over-run */
	if (left > q_pos)
		if (q_pos + right < WmWM)
			return;
		else
			left = q_pos;
	if ((limit = q_len - q_pos - WORDSIZE_MIN) < (int)right)
		if ((int)left + limit < WmWM)
			return;
		else
			right = limit;

	/* given the offset of the hit, find i.d. number of the database sequence */
	cbyte = c - MPPTR c_seq - 1; /* offset of the first byte of the 8-mer hit */
	c_pad = cseq_beg[MPPTR current] & BITMASK(NBPN);

	/* return if W-base hit crosses a database sequence boundary */
	if ((cbyte == 0 && right < WmWM)
		|| (cbyte > c_end-2
	    	&& (cbyte == c_end
	    		|| (cbyte == c_end-1 && (c_pad != 0 || left < WmWM)) ) ) )
		return;

	MPPTR numhits++;

	diag = cbyte*(CHAR_BIT/NBPN) - q_pos;
	for (x = 0; x < MPPTR diag_top; ++x) {
		if (MPPTR diag_level[x].diag == diag)
			if (MPPTR diag_level[x].pos <= q_pos) {
				diagentry = x;
				goto GotDiagEntry;
			}
			else
				return;
	}
	if (MPPTR diag_top >= DIAG_MAX)
		fatal("too many matches; raise DIAG_MAX and recompile,\n  or query with a shorter sequence");

	MPPTR diag_top = x+1;
	MPPTR diag_level[diagentry = x].diag = diag;
GotDiagEntry:

	/* extend to the left */
	d = dlim = NULL;
	q_beg = q = qq_seq + q_pos;
	sum = score = 0;
	x = minusX;
	cbp = MPPTR c_seq + cbyte;
	cend = MPPTR c_seq + c_end;
	/* Optimizing compilers are having difficulty with these loops */
	while (q > qq_seq) {
		if (d == dlim) {
			if (cbp <= MPPTR c_seq)
				break;
			d = (dlim = &xray[*--cbp][0]) + DIM2(xray);
		}
		if (*--d != *--q) {
			if ((sum += n) >= x)
				continue;
			break;
		}
		if ((sum += m) > 0) {
			score += sum;
			sum = 0;
			q_beg = q;
		}
	}

	/* extend to the right */
	cbp = MPPTR c_seq + cbyte + 1;
	d = dlim;
	q_end = q = qq_seq + q_pos + WORDSIZE_MIN;
	sum = 0;
	while (q < qq_seq + q_len) {
		if (d == dlim) {
			if (cbp >= cend)
				break;
			dlim = (d = &xray[*++cbp][0]) + DIM2(xray);
			if (cbp == cend && c_pad > 0)
				dlim = d + c_pad;
		}
		if (*d++ != *q++) {
			if ((sum += n) >= x)
				continue;
			break;
		}
		if ((sum += m) > 0) {
			score += sum;
			sum = 0;
			q_end = q;
		}
	}
	MPPTR diag_level[diagentry].pos = q_end - qq_seq;
	if (score + MtWM < S) {
		++MPPTR numfailed;
		return;
	}

	{
		struct hsp	h;
		h.score = score + MtWM;
		h.seq_nbr = MPPTR current;
		h.len = q_end - q_beg;
		h.q_pos = q_beg - qq_seq + 1;
		if (qq_seq == qq_rev)
			h.q_pos = -h.q_pos;
		h.s_pos = cbyte*(CHAR_BIT/NBPN) - (q_pos - (q_beg-qq_seq));
		ckoverlap(MPC &h);
	}
}
