static void	extend_x PROTO((MPPROTOC char far *s, far_mempint pos));

/* -----------------------   search a database sequence   --------------------*/
static void
search_x(MP)
	MPDEF
{
	register STATE	*state;
	register dfa_accept	*ap;
	register char far *s;

	state = dfa_state0(dp);
	s = MPPTR s_seq;

	for (;;) {
		/*
			Machine "Accepts" when a word hit is encountered,
			and "Halts" on the null terminator or any other non-alphabetic
			character encountered
		*/
		while (!IsAccepting(state = state->next[*s++]))
			;

		/* Has finite-state machine halted (e.g., end of sequence)? */
		if ((ap = (dfa_accept *)Unmunged(state)) == NULL)
			break; /* yes */

		/* Found at least one hit... */

		/* Does list of hit positions contain more than one item? */
		if (!IsMunged(state = ap->ptr.retState))
			ap = ap->val.next; /* yes, ap->val.next points to head of list */

		/* Always Unmunge state, to avoid a costly jump if it isn't */
		state = (STATE *)Unmunged(state);

		/* Extend each hit in the linked list */
		do
			extend_x(MPC s, (mempint)ap->val.patID);
		while (!IsMunged(ap = ap->ptr.chain));
	}

	{
		register DIAG_TYPE	*diag_level;
		register int	*diag_top, *diag_stack;
		register int	frame;
		seq_data	*sdp;

		for (frame=minframe; frame<maxframe; ++frame) {
			sdp = &MPPTR sd[frame];
			diag_level = sdp->diag_level;
			diag_top = sdp->diag_top;
			diag_stack = sdp->diag_stack;
			while (diag_top > diag_stack)
				diag_level[*--diag_top] = 0;
			sdp->diag_top = diag_top;
		}
	}
}

/* extend_x - extend a word-sized hit to a longer match */
static void
extend_x(MPC s_pos, pos)
	MPDEF
	register char far	*s_pos;
	register mempint	pos;
{
	register char	*q, *s;
	register Int4	sum, x;
	register Int4	score;
	register char	*q_pos;
	register char	*q_beg, *q_end;
	register seq_data	*sdp;
	Int4	diag;
	char	*q_seq;

	++MPPTR numhits;

	sdp = &MPPTR sd[pos & 7];
	pos >>= 3;
	if (sdp->diag_level[diag = (s_pos - MPPTR s_seq) - pos] > pos)
		return;

	q_pos = (q_seq = sdp->seq) + pos;

	/* Find where positive scoring starts & ends within the word hit */
	q_beg = q = q_pos - W;
	s = s_pos - W;
	score = sum = 0;
	while (q < q_pos) {
		if ((sum += matrx[*q++][*s++]) > score) {
			q_end = q;
			score = sum;
			continue;
		}
		if (sum <= 0) {
			q_beg = q;
			score = sum = 0;
		}
	}

	x = sdp->minusX; /* important! */

	/* extend to the right */
	s = s_pos + ((q = q_end) - q_pos);
	sum = 0;
	do
		if ((sum += matrx[*q++][*s++]) > 0) {
			score += sum;
			sum = 0;
			q_end = q;
		}
	while (sum >= x); /* fails on null terminator of either sequence */

	/* extend to the left */
	s = s_pos - (q_pos - (q = q_beg));
	sum = 0;
	do
		if ((sum += matrx[*--q][*--s]) > 0) {
			score += sum;
			sum = 0;
			q_beg = q;
		}
	while (sum >= x); /* fails on null prefix of either sequence */

	/* If score is reportable, examine in more detail... */
	if (score >= sdp->S) {
		/* extend left to end of diagonal (this time ignoring X) */
		s = s_pos - (q_pos - (q = q_beg));
		sum = 0;
		do
			if ((sum += matrx[*--q][*--s]) > 0) {
				score += sum;
				sum = 0;
				q_beg = q;
			}
		while (sum > SCORE_MIN/4); /* fails on null prefix of either sequence */

		/* extend right to end of diagonal (this time ignoring X) */
		s = s_pos + ((q = q_end) - q_pos);
		sum = 0;
		do
			if ((sum += matrx[*q++][*s++]) > 0) {
				score += sum;
				sum = 0;
				q_end = q;
			}
		while (sum > SCORE_MIN/4); /* fails on null terminator */

		/* Record the HSP */
		{
			struct hsp	h;

			h.seq_nbr = MPPTR current;
			h.len = q_end - q_beg;
			h.frame = sdp->frame;
			h.q_pos = q_beg - q_seq;
			h.s_pos = s_pos - MPPTR s_seq - (q_pos - q_beg);
			h.score = score;
			ckoverlap(MPC &h);
		}
	}
	else
		++MPPTR numfailed;

	if (sdp->diag_level[diag] == 0)
		*(sdp->diag_top++) = diag;
	sdp->diag_level[diag] = (q_end - q_seq) + W;
}
