/*************************************************************************
*                                                                        *
*                             COPYRIGHT NOTICE                           *
*                                                                        *
* This software/database is "United States Government Work" under the	 *
* terms of the United States Copyright Act.  It was written as part of	 *
* the author's official duties as a Government employee and thus cannot	 *
* be copyrighted.  This software/database is freely available to the	 *
* public for use without a copyright notice.  Restrictions cannot be	 *
* placed on its present or future use.                                   *
*                                                                        *
* Although all reasonable efforts have been taken to insure the accuracy *
* and reliability of the software and data, the National Library of      *
* Medicine (NLM) and the U.S. Government does not and cannot warrant the *
* performance or results that may be obtained by using this software or  *
* data.  The NLM and the U.S. Government disclaims all warranties as to  *
* performance, merchantability or fitness for any particular purpose.    *
*                                                                        *
* In any work or product derived from this material, proper attribution  *
* of the author as the source of the software or data would be           *
* appreciated.                                                           *
*                                                                        *
*************************************************************************/

/*
 *  Allocate a shared memory segment; give other processes access.
 */
#include <ncbi.h>

#ifdef SYSV_SHM_AVAIL

#include <sys/ipc.h>
#include <sys/shm.h>

char *
shmalloc(theKey, mode, size)
	key_t	theKey;
	int		mode;
    size_t	size;
{
	char	*module = "shmalloc";
    extern char *shmat();
    int     shmid;
    struct shmid_ds shmds;
    char    *cp;

    shmid = shmget(theKey, size, IPC_CREAT);
    if (shmid == -1) {
        perror(module);
        return NULL;
    }

    shmctl(shmid, IPC_STAT, &shmds);
    shmds.shm_perm.mode |= mode;
    shmctl(shmid, IPC_SET, &shmds);

	/* Lock the segment in core */
	shmctl(shmid, SHM_LOCK, &shmds);

    cp = shmat(shmid, NULL, 0);
    if (cp == NULL || cp == (char *)-1) {
        perror(module);
        return NULL;
    }
	return cp;
}


int
shmexists(theKey, size)
	key_t	theKey;
	unsigned int	size;
{
	int		shmid;

	shmid = shmget(theKey, size, 0);
	if (shmid == -1)
		return (1==0);
	return (1==1);
}

#endif /* SYSV_SHM_AVAIL */
